package com.retail_store.order_management.model;


import com.retail_store.order_management.constants.Category;
import com.retail_store.order_management.constants.Color;
import com.retail_store.order_management.constants.Gender;
import com.retail_store.order_management.constants.Size;
import lombok.AccessLevel;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.Map;

@Getter
@AllArgsConstructor
public class Product {
    private String id;
    private String name;
    private Category category;
    private Color color;
    private double price;
    private Gender idealFor;
    private Map<Size, Integer> size;

    public String getProductSizeAvailability() {
        return size.entrySet().stream().map(entry -> entry.getKey() + " - " + entry.getValue()).reduce("", (sizes, size) -> sizes + "\n" + size.toString());
    }

    public boolean outOfStock() {
        return size.entrySet().stream().allMatch(entry -> entry.getValue() <= 0);
    }

    public String showSizes() {
        return outOfStock() ? "* Out of Stock *" : "Sizes available : " + size.entrySet().stream().map(size -> size.getKey() + "-" + size.getValue()).reduce((sizes, size) -> sizes + ", " + size).get();
    }

    public String productShowcaseInMenu() {
        return getName() + "\t" + getPrice() + "\n   " + getColor() + "\t" + showSizes();
    }

    @Override
    public String toString() {
        return "Product{" +
                "id='" + id + '\'' +
                ", name='" + name + '\'' +
                ", category=" + category +
                ", color=" + color +
                ", price=" + price +
                ", idealFor=" + idealFor +
                ", size=" + getProductSizeAvailability() +
                '}';
    }
}